'use strict';

Object.defineProperty(exports, '__esModule', {
  value: true
});
exports['default'] = immutableStateInvariantMiddleware;

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { 'default': obj }; }

var _invariant = require('invariant');

var _invariant2 = _interopRequireDefault(_invariant);

var _isImmutable = require('./isImmutable');

var _isImmutable2 = _interopRequireDefault(_isImmutable);

var _copyState = require('./copyState');

var _copyState2 = _interopRequireDefault(_copyState);

var _wasMutated = require('./wasMutated');

var _wasMutated2 = _interopRequireDefault(_wasMutated);

var BETWEEN_DISPATCHES_MESSAGE = ['A state mutation was detected between dispatches, in the path `%s`.', 'This may cause incorrect behavior.', '(http://rackt.github.io/redux/docs/Troubleshooting.html#never-mutate-reducer-arguments)'].join(' ');

var INSIDE_DISPATCH_MESSAGE = ['A state mutation was detected inside a dispatch, in the path: `%s`.', 'Take a look at the reducer(s) handling the action %s.', '(http://rackt.github.io/redux/docs/Troubleshooting.html#never-mutate-reducer-arguments)'].join(' ');

function immutableStateInvariantMiddleware() {
  var isImmutable = arguments.length <= 0 || arguments[0] === undefined ? _isImmutable2['default'] : arguments[0];

  return function (_ref) {
    var getState = _ref.getState;

    var lastStateRef = getState();
    var lastStateCopy = (0, _copyState2['default'])(lastStateRef, isImmutable);

    var result = undefined;
    return function (next) {
      return function (action) {
        var stateRef = getState();

        result = (0, _wasMutated2['default'])(lastStateRef, lastStateCopy, isImmutable);
        (0, _invariant2['default'])(!result.wasMutated, BETWEEN_DISPATCHES_MESSAGE, (result.path || []).join('.'));

        var stateCopy = (0, _copyState2['default'])(stateRef, isImmutable);
        var dispatchedAction = next(action);
        lastStateRef = getState();

        result = (0, _wasMutated2['default'])(stateRef, stateCopy, isImmutable);
        (0, _invariant2['default'])(!result.wasMutated, INSIDE_DISPATCH_MESSAGE, (result.path || []).join('.'), JSON.stringify(action));

        lastStateCopy = (0, _copyState2['default'])(lastStateRef, isImmutable);
        return dispatchedAction;
      };
    };
  };
}

module.exports = exports['default'];